{
  "cells": [
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "\n# Meteogram\n\nPlots time series data as a meteogram.\n"
      ]
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {
        "collapsed": false
      },
      "outputs": [],
      "source": [
        "import datetime as dt\n\nimport matplotlib as mpl\nimport matplotlib.pyplot as plt\nfrom matplotlib.ticker import MultipleLocator\nimport numpy as np\n\nfrom metpy.calc import dewpoint_from_relative_humidity\nfrom metpy.cbook import get_test_data\nfrom metpy.plots import add_metpy_logo\nfrom metpy.units import units\n\n\ndef calc_mslp(t, p, h):\n    return p * (1 - (0.0065 * h) / (t + 0.0065 * h + 273.15)) ** (-5.257)\n\n\n# Make meteogram plot\nclass Meteogram:\n    \"\"\" Plot a time series of meteorological data from a particular station as a\n    meteogram with standard variables to visualize, including thermodynamic,\n    kinematic, and pressure. The functions below control the plotting of each\n    variable.\n    TO DO: Make the subplot creation dynamic so the number of rows is not\n    static as it is currently. \"\"\"\n\n    def __init__(self, fig, dates, probeid, time=None, axis=0):\n        \"\"\"\n        Required input:\n            fig: figure object\n            dates: array of dates corresponding to the data\n            probeid: ID of the station\n        Optional Input:\n            time: Time the data is to be plotted\n            axis: number that controls the new axis to be plotted (FOR FUTURE)\n        \"\"\"\n        if not time:\n            time = dt.datetime.utcnow()\n        self.start = dates[0]\n        self.fig = fig\n        self.end = dates[-1]\n        self.axis_num = 0\n        self.dates = mpl.dates.date2num(dates)\n        self.time = time.strftime('%Y-%m-%d %H:%M UTC')\n        self.title = f'Latest Ob Time: {self.time}\\nProbe ID: {probeid}'\n\n    def plot_winds(self, ws, wd, wsmax, plot_range=None):\n        \"\"\"\n        Required input:\n            ws: Wind speeds (knots)\n            wd: Wind direction (degrees)\n            wsmax: Wind gust (knots)\n        Optional Input:\n            plot_range: Data range for making figure (list of (min,max,step))\n        \"\"\"\n        # PLOT WIND SPEED AND WIND DIRECTION\n        self.ax1 = fig.add_subplot(4, 1, 1)\n        ln1 = self.ax1.plot(self.dates, ws, label='Wind Speed')\n        self.ax1.fill_between(self.dates, ws, 0)\n        self.ax1.set_xlim(self.start, self.end)\n        ymin, ymax, ystep = plot_range if plot_range else (0, 20, 2)\n        self.ax1.set_ylabel('Wind Speed (knots)', multialignment='center')\n        self.ax1.set_ylim(ymin, ymax)\n        self.ax1.yaxis.set_major_locator(MultipleLocator(ystep))\n        self.ax1.grid(which='major', axis='y', color='k', linestyle='--', linewidth=0.5)\n        ln2 = self.ax1.plot(self.dates, wsmax, '.r', label='3-sec Wind Speed Max')\n\n        ax7 = self.ax1.twinx()\n        ln3 = ax7.plot(self.dates, wd, '.k', linewidth=0.5, label='Wind Direction')\n        ax7.set_ylabel('Wind\\nDirection\\n(degrees)', multialignment='center')\n        ax7.set_ylim(0, 360)\n        ax7.set_yticks(np.arange(45, 405, 90))\n        ax7.set_yticklabels(['NE', 'SE', 'SW', 'NW'])\n        lines = ln1 + ln2 + ln3\n        labs = [line.get_label() for line in lines]\n        ax7.xaxis.set_major_formatter(mpl.dates.DateFormatter('%d/%H UTC'))\n        ax7.legend(lines, labs, loc='upper center',\n                   bbox_to_anchor=(0.5, 1.2), ncol=3, prop={'size': 12})\n\n    def plot_thermo(self, t, td, plot_range=None):\n        \"\"\"\n        Required input:\n            T: Temperature (deg F)\n            TD: Dewpoint (deg F)\n        Optional Input:\n            plot_range: Data range for making figure (list of (min,max,step))\n        \"\"\"\n        # PLOT TEMPERATURE AND DEWPOINT\n        ymin, ymax, ystep = plot_range if plot_range else (10, 90, 5)\n        self.ax2 = fig.add_subplot(4, 1, 2, sharex=self.ax1)\n        ln4 = self.ax2.plot(self.dates, t, 'r-', label='Temperature')\n        self.ax2.fill_between(self.dates, t, td, color='r')\n\n        self.ax2.set_ylabel('Temperature\\n(F)', multialignment='center')\n        self.ax2.grid(which='major', axis='y', color='k', linestyle='--', linewidth=0.5)\n        self.ax2.set_ylim(ymin, ymax)\n        self.ax2.yaxis.set_major_locator(MultipleLocator(ystep))\n\n        ln5 = self.ax2.plot(self.dates, td, 'g-', label='Dewpoint')\n        self.ax2.fill_between(self.dates, td, self.ax2.get_ylim()[0], color='g')\n\n        ax_twin = self.ax2.twinx()\n        ax_twin.set_ylim(ymin, ymax)\n        ax_twin.yaxis.set_major_locator(MultipleLocator(ystep))\n        lines = ln4 + ln5\n        labs = [line.get_label() for line in lines]\n        ax_twin.xaxis.set_major_formatter(mpl.dates.DateFormatter('%d/%H UTC'))\n\n        self.ax2.legend(lines, labs, loc='upper center',\n                        bbox_to_anchor=(0.5, 1.2), ncol=2, prop={'size': 12})\n\n    def plot_rh(self, rh, plot_range=None):\n        \"\"\"\n        Required input:\n            RH: Relative humidity (%)\n        Optional Input:\n            plot_range: Data range for making figure (list of (min,max,step))\n        \"\"\"\n        # PLOT RELATIVE HUMIDITY\n        ymin, ymax, ystep = plot_range if plot_range else (0, 100, 5)\n        self.ax3 = fig.add_subplot(4, 1, 3, sharex=self.ax1)\n        self.ax3.plot(self.dates, rh, 'g-', label='Relative Humidity')\n        self.ax3.legend(loc='upper center', bbox_to_anchor=(0.5, 1.22), prop={'size': 12})\n        self.ax3.grid(which='major', axis='y', color='k', linestyle='--', linewidth=0.5)\n        self.ax3.set_ylim(ymin, ymax)\n        self.ax3.yaxis.set_major_locator(MultipleLocator(ystep))\n\n        self.ax3.fill_between(self.dates, rh, self.ax3.get_ylim()[0], color='g')\n        self.ax3.set_ylabel('Relative Humidity\\n(%)', multialignment='center')\n        self.ax3.xaxis.set_major_formatter(mpl.dates.DateFormatter('%d/%H UTC'))\n        axtwin = self.ax3.twinx()\n        axtwin.set_ylim(ymin, ymax)\n        axtwin.yaxis.set_major_locator(MultipleLocator(ystep))\n\n    def plot_pressure(self, p, plot_range=None):\n        \"\"\"\n        Required input:\n            P: Mean Sea Level Pressure (hPa)\n        Optional Input:\n            plot_range: Data range for making figure (list of (min,max,step))\n        \"\"\"\n        # PLOT PRESSURE\n        ymin, ymax, ystep = plot_range if plot_range else (970, 1030, 4)\n        self.ax4 = fig.add_subplot(4, 1, 4, sharex=self.ax1)\n        self.ax4.plot(self.dates, p, 'm', label='Mean Sea Level Pressure')\n        self.ax4.set_ylabel('Mean Sea\\nLevel Pressure\\n(mb)', multialignment='center')\n        self.ax4.set_ylim(ymin, ymax)\n        self.ax4.yaxis.set_major_locator(MultipleLocator(ystep))\n\n        axtwin = self.ax4.twinx()\n        axtwin.set_ylim(ymin, ymax)\n        axtwin.yaxis.set_major_locator(MultipleLocator(ystep))\n        axtwin.fill_between(self.dates, p, axtwin.get_ylim()[0], color='m')\n        axtwin.xaxis.set_major_formatter(mpl.dates.DateFormatter('%d/%H UTC'))\n\n        self.ax4.legend(loc='upper center', bbox_to_anchor=(0.5, 1.2), prop={'size': 12})\n        self.ax4.grid(which='major', axis='y', color='k', linestyle='--', linewidth=0.5)\n        # OTHER OPTIONAL AXES TO PLOT\n        # plot_irradiance\n        # plot_precipitation\n\n\n# set the starttime and endtime for plotting, 24 hour range\nendtime = dt.datetime(2016, 3, 31, 22, 0, 0, 0)\nstarttime = endtime - dt.timedelta(hours=24)\n\n# Height of the station to calculate MSLP\nhgt_example = 292.\n\n\n# Parse dates from .csv file, knowing their format as a string and convert to datetime\ndef parse_date(date):\n    return dt.datetime.strptime(date.decode('ascii'), '%Y-%m-%d %H:%M:%S')\n\n\ntestdata = np.genfromtxt(get_test_data('timeseries.csv', False), names=True, dtype=None,\n                         usecols=list(range(1, 8)),\n                         converters={'DATE': parse_date}, delimiter=',')\n\n# Temporary variables for ease\ntemp = testdata['T']\npressure = testdata['P']\nrh = testdata['RH']\nws = testdata['WS']\nwsmax = testdata['WSMAX']\nwd = testdata['WD']\ndate = testdata['DATE']\n\n# ID For Plotting on Meteogram\nprobe_id = '0102A'\n\ndata = {'wind_speed': (np.array(ws) * units('m/s')).to(units('knots')),\n        'wind_speed_max': (np.array(wsmax) * units('m/s')).to(units('knots')),\n        'wind_direction': np.array(wd) * units('degrees'),\n        'dewpoint': dewpoint_from_relative_humidity((np.array(temp) * units.degC).to(units.K),\n                                                    np.array(rh) / 100.).to(units('degF')),\n        'air_temperature': (np.array(temp) * units('degC')).to(units('degF')),\n        'mean_slp': calc_mslp(np.array(temp), np.array(pressure), hgt_example) * units('hPa'),\n        'relative_humidity': np.array(rh), 'times': np.array(date)}\n\nfig = plt.figure(figsize=(20, 16))\nadd_metpy_logo(fig, 250, 180)\nmeteogram = Meteogram(fig, data['times'], probe_id)\nmeteogram.plot_winds(data['wind_speed'], data['wind_direction'], data['wind_speed_max'])\nmeteogram.plot_thermo(data['air_temperature'], data['dewpoint'])\nmeteogram.plot_rh(data['relative_humidity'])\nmeteogram.plot_pressure(data['mean_slp'])\nfig.subplots_adjust(hspace=0.5)\nplt.show()"
      ]
    }
  ],
  "metadata": {
    "kernelspec": {
      "display_name": "Python 3",
      "language": "python",
      "name": "python3"
    },
    "language_info": {
      "codemirror_mode": {
        "name": "ipython",
        "version": 3
      },
      "file_extension": ".py",
      "mimetype": "text/x-python",
      "name": "python",
      "nbconvert_exporter": "python",
      "pygments_lexer": "ipython3",
      "version": "3.10.12"
    }
  },
  "nbformat": 4,
  "nbformat_minor": 0
}